<?php

namespace OpenlayersDoctrine\Common\Annotations;

/**
 * Simple Annotation Reader.
 *
 * This annotation reader is intended to be used in projects where you have
 * full-control over all annotations that are available.
 *
 * @since 2.2
 *
 * @author Johannes M. Schmitt <schmittjoh@gmail.com>
 * @author Fabio B. Silva <fabio.bat.silva@gmail.com>
 */
class SimpleAnnotationReader implements Reader {

  /**
   * FIX - insert comment here.
   *
   * @var DocParser
   */
  private $parser;

  /**
   * Constructor.
   *
   * Initializes a new SimpleAnnotationReader.
   */
  public function __construct() {
    $this->parser = new DocParser();
    $this->parser->setIgnoreNotImportedAnnotations(TRUE);
  }

  /**
   * Adds a namespace in which we will look for annotations.
   *
   * @param string $namespace
   *   FIX - insert comment here.
   */
  public function addNamespace($namespace) {
    $this->parser->addNamespace($namespace);
  }

  /**
   * FIX - insert comment here.
   */
  public function getClassAnnotations(\ReflectionClass $class) {
    return $this->parser->parse($class->getDocComment(), 'class ' . $class->getName());
  }

  /**
   * FIX - insert comment here.
   */
  public function getMethodAnnotations(\ReflectionMethod $method) {
    return $this->parser->parse($method->getDocComment(), 'method ' . $method->getDeclaringClass()->name . '::' . $method->getName() . '()');
  }

  /**
   * FIX - insert comment here.
   */
  public function getPropertyAnnotations(\ReflectionProperty $property) {
    return $this->parser->parse($property->getDocComment(), 'property ' . $property->getDeclaringClass()->name . '::$' . $property->getName());
  }

  /**
   * FIX - insert comment here.
   */
  public function getClassAnnotation(\ReflectionClass $class, $annotationName) {
    foreach ($this->getClassAnnotations($class) as $annot) {
      if ($annot instanceof $annotationName) {
        return $annot;
      }
    }

    return NULL;
  }

  /**
   * FIX - insert comment here.
   */
  public function getMethodAnnotation(\ReflectionMethod $method, $annotationName) {
    foreach ($this->getMethodAnnotations($method) as $annot) {
      if ($annot instanceof $annotationName) {
        return $annot;
      }
    }

    return NULL;
  }

  /**
   * FIX - insert comment here.
   */
  public function getPropertyAnnotation(\ReflectionProperty $property, $annotationName) {
    foreach ($this->getPropertyAnnotations($property) as $annot) {
      if ($annot instanceof $annotationName) {
        return $annot;
      }
    }

    return NULL;
  }

}
